package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.CleanupJobStatus;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.mockito.internal.stubbing.answers.AnswersWithDelay;

import static org.junit.Assert.assertEquals;

public class CleanupServiceTest {
    private BlurringDao blurringDao;

    @Before
    public void setUp() {
        blurringDao = Mockito.mock(BlurringDao.class);
    }

    @Test
    public void doCleanup_WhenExecutionTimeIsExceeded_statusIsUnfinished() {
        // Arrange

        // Max execution time is 1 seconds, batch size 150
        CleanupService cleanupService = new CleanupService(blurringDao, 1, 150);
        // 10000 rows to delete in total
        Mockito.when(blurringDao.getInactiveBlurringsCount()).thenReturn(10000);
        // Each back takes 500 milliseconds
        Mockito.when(blurringDao.deleteRowsInBatch(Mockito.anyInt())).thenAnswer(new AnswersWithDelay(500, invocation -> invocation.getArgument(0)));

        // Act
        cleanupService.cleanUpBlurrings();

        // Assert
        CleanupJobStatus status = cleanupService.getStatus();
        assertEquals(300, status.rowsDeleted);
        assertEquals(CleanupJobStatus.ExecutionStatus.NOT_FINISHED, status.finished);
    }

    @Test
    public void doCleanup_WhenAllRowsAreDeleted_statusIsFinished() {
        // Arrange

        // Max execution time is 1 seconds, batch size 100
        CleanupService cleanupService = new CleanupService(blurringDao, 1, 100);
        // 1000 rows to delete in total
        Mockito.when(blurringDao.getInactiveBlurringsCount()).thenReturn(1000);
        // Each back takes 100 milliseconds
        Mockito.when(blurringDao.deleteRowsInBatch(Mockito.anyInt())).thenAnswer(new AnswersWithDelay(100, invocation -> invocation.getArgument(0)));

        // Act
        cleanupService.cleanUpBlurrings();

        // Assert
        CleanupJobStatus status = cleanupService.getStatus();
        assertEquals(1000, status.rowsDeleted);
        assertEquals(CleanupJobStatus.ExecutionStatus.FINISHED, status.finished);
    }

}
